"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = CodeMirror;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _forEach2 = _interopRequireDefault(require("lodash/forEach"));

var _react = _interopRequireWildcard(require("react"));

var _createEditor = _interopRequireDefault(require("../../util/create-editor"));

var _computeScrollHeight = _interopRequireDefault(require("../../util/compute-scroll-height"));

var _wrapper = _interopRequireDefault(require("./wrapper"));

var _excluded = ["conf", "convertValueToDisplay", "convertValueFromDisplay", "minLines", "maxLines", "value", "onChange"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function convertValue(displayValue, convert) {
  return convert ? convert(displayValue) || '' : displayValue;
}

function usePrevious(value) {
  var ref = (0, _react.useRef)(null);
  (0, _react.useEffect)(function () {
    ref.current = value;
  });
  return ref.current;
}
/**
 * CodeMirror
 */


function CodeMirror(_ref) {
  var conf = _ref.conf,
      convertValueToDisplay = _ref.convertValueToDisplay,
      convertValueFromDisplay = _ref.convertValueFromDisplay,
      _ref$minLines = _ref.minLines,
      minLines = _ref$minLines === void 0 ? 6 : _ref$minLines,
      _ref$maxLines = _ref.maxLines,
      maxLines = _ref$maxLines === void 0 ? 30 : _ref$maxLines,
      _ref$value = _ref.value,
      value = _ref$value === void 0 ? '' : _ref$value,
      onChange = _ref.onChange,
      props = (0, _objectWithoutProperties2.default)(_ref, _excluded);

  var _useState = (0, _react.useState)(null),
      _useState2 = (0, _slicedToArray2.default)(_useState, 2),
      stateEditorDom = _useState2[0],
      setStateEditorDom = _useState2[1];

  var _useState3 = (0, _react.useState)(null),
      _useState4 = (0, _slicedToArray2.default)(_useState3, 2),
      stateEditor = _useState4[0],
      setStateEditor = _useState4[1];

  var _useState5 = (0, _react.useState)(convertValue(value, convertValueToDisplay)),
      _useState6 = (0, _slicedToArray2.default)(_useState5, 2),
      stateDisplayValue = _useState6[0],
      setStateDisplayValue = _useState6[1];

  var prevValue = usePrevious(value);
  var handleChange = (0, _react.useCallback)(function (editor) {
    var newValue = editor.getValue();
    setStateDisplayValue(newValue);

    if (onChange) {
      onChange(convertValue(newValue, convertValueFromDisplay));
    }
  }, [convertValueFromDisplay, onChange]);
  (0, _react.useEffect)(function () {
    if (!stateEditorDom || stateEditor) {
      return;
    }

    var editor = (0, _createEditor.default)(stateEditorDom, stateDisplayValue, conf); // TODO 要不要更新 conf？

    (0, _forEach2.default)(conf === null || conf === void 0 ? void 0 : conf.on, function (fn, k) {
      return editor.on(k, fn);
    });
    setStateEditor(editor);
  }, [conf, stateEditorDom, stateEditor, stateDisplayValue, setStateEditor]); // onChange 变化则更新之，需要解除旧有的

  (0, _react.useEffect)(function () {
    if (!stateEditor) {
      return;
    }

    stateEditor.on('change', handleChange);
    return function () {
      return stateEditor.off('change', handleChange);
    };
  }, [handleChange, stateEditor]);
  (0, _react.useEffect)(function () {
    if (!stateEditor) {
      return;
    }

    var newDisplayValue = convertValue(value, convertValueToDisplay);

    if (prevValue !== value && stateDisplayValue !== newDisplayValue) {
      setStateDisplayValue(newDisplayValue);
      stateEditor.setValue(newDisplayValue);
    }
  }, [prevValue, value, stateDisplayValue, convertValueToDisplay, stateEditor]);
  (0, _react.useEffect)(function () {
    // CodeMirror 未提供 destroy 方法，只需要解除实例的引用即可
    return function () {
      return setStateEditor(null);
    };
  }, [setStateEditor]);
  return /*#__PURE__*/_react.default.createElement(_wrapper.default, (0, _extends2.default)({
    ref: setStateEditorDom
  }, _objectSpread(_objectSpread({}, props), {}, {
    minHeight: (0, _computeScrollHeight.default)(minLines),
    maxHeight: (0, _computeScrollHeight.default)(maxLines)
  })));
}