import _extends from "@babel/runtime/helpers/esm/extends";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
var _excluded = ["conf", "convertValueToDisplay", "convertValueFromDisplay", "minLines", "maxLines", "value", "onChange"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

import _forEach from 'lodash/forEach';
import React, { useRef, useState, useCallback, useEffect } from 'react';
import createEditor from '../../util/create-editor';
import computeScrollHeight from '../../util/compute-scroll-height';
import Wrapper from './wrapper';

function convertValue(displayValue, convert) {
  return convert ? convert(displayValue) || '' : displayValue;
}

function usePrevious(value) {
  var ref = useRef(null);
  useEffect(function () {
    ref.current = value;
  });
  return ref.current;
}
/**
 * CodeMirror
 */


export default function CodeMirror(_ref) {
  var conf = _ref.conf,
      convertValueToDisplay = _ref.convertValueToDisplay,
      convertValueFromDisplay = _ref.convertValueFromDisplay,
      _ref$minLines = _ref.minLines,
      minLines = _ref$minLines === void 0 ? 6 : _ref$minLines,
      _ref$maxLines = _ref.maxLines,
      maxLines = _ref$maxLines === void 0 ? 30 : _ref$maxLines,
      _ref$value = _ref.value,
      value = _ref$value === void 0 ? '' : _ref$value,
      onChange = _ref.onChange,
      props = _objectWithoutProperties(_ref, _excluded);

  var _useState = useState(null),
      _useState2 = _slicedToArray(_useState, 2),
      stateEditorDom = _useState2[0],
      setStateEditorDom = _useState2[1];

  var _useState3 = useState(null),
      _useState4 = _slicedToArray(_useState3, 2),
      stateEditor = _useState4[0],
      setStateEditor = _useState4[1];

  var _useState5 = useState(convertValue(value, convertValueToDisplay)),
      _useState6 = _slicedToArray(_useState5, 2),
      stateDisplayValue = _useState6[0],
      setStateDisplayValue = _useState6[1];

  var prevValue = usePrevious(value);
  var handleChange = useCallback(function (editor) {
    var newValue = editor.getValue();
    setStateDisplayValue(newValue);

    if (onChange) {
      onChange(convertValue(newValue, convertValueFromDisplay));
    }
  }, [convertValueFromDisplay, onChange]);
  useEffect(function () {
    if (!stateEditorDom || stateEditor) {
      return;
    }

    var editor = createEditor(stateEditorDom, stateDisplayValue, conf); // TODO 要不要更新 conf？

    _forEach(conf === null || conf === void 0 ? void 0 : conf.on, function (fn, k) {
      return editor.on(k, fn);
    });

    setStateEditor(editor);
  }, [conf, stateEditorDom, stateEditor, stateDisplayValue, setStateEditor]); // onChange 变化则更新之，需要解除旧有的

  useEffect(function () {
    if (!stateEditor) {
      return;
    }

    stateEditor.on('change', handleChange);
    return function () {
      return stateEditor.off('change', handleChange);
    };
  }, [handleChange, stateEditor]);
  useEffect(function () {
    if (!stateEditor) {
      return;
    }

    var newDisplayValue = convertValue(value, convertValueToDisplay);

    if (prevValue !== value && stateDisplayValue !== newDisplayValue) {
      setStateDisplayValue(newDisplayValue);
      stateEditor.setValue(newDisplayValue);
    }
  }, [prevValue, value, stateDisplayValue, convertValueToDisplay, stateEditor]);
  useEffect(function () {
    // CodeMirror 未提供 destroy 方法，只需要解除实例的引用即可
    return function () {
      return setStateEditor(null);
    };
  }, [setStateEditor]);
  return /*#__PURE__*/React.createElement(Wrapper, _extends({
    ref: setStateEditorDom
  }, _objectSpread(_objectSpread({}, props), {}, {
    minHeight: computeScrollHeight(minLines),
    maxHeight: computeScrollHeight(maxLines)
  })));
}